@@ -11,18 +11,21 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import pretend
+import io
 
+import pretend
+import pytest
+
+from twine import commands
+from twine import package as package_file
 from twine.commands import check
 
 
 class TestWarningStream:
-
     def setup(self):
         self.stream = check._WarningStream()
         self.stream.output = pretend.stub(
-            write=pretend.call_recorder(lambda a: None),
-            getvalue=lambda: "result",
+            write=pretend.call_recorder(lambda a: None), getvalue=lambda: "result",
         )
 
     def test_write_match(self):
@@ -35,101 +38,122 @@ class TestWarningStream:
     def test_write_nomatch(self):
         self.stream.write("this does not match")
 
-        assert self.stream.output.write.calls == [
-            pretend.call("this does not match")
-        ]
+        assert self.stream.output.write.calls == [pretend.call("this does not match")]
 
     def test_str_representation(self):
         assert str(self.stream) == "result"
 
 
 def test_check_no_distributions(monkeypatch):
-    stream = check.StringIO()
+    stream = io.StringIO()
 
-    monkeypatch.setattr(check, "_find_dists", lambda a: [])
+    monkeypatch.setattr(commands, "_find_dists", lambda a: [])
 
-    assert not check.check("dist/*", output_stream=stream)
+    assert not check.check(["dist/*"], output_stream=stream)
     assert stream.getvalue() == "No files to check.\n"
 
 
 def test_check_passing_distribution(monkeypatch):
-    renderer = pretend.stub(
-        render=pretend.call_recorder(lambda *a, **kw: "valid")
+    renderer = pretend.stub(render=pretend.call_recorder(lambda *a, **kw: "valid"))
+    package = pretend.stub(
+        metadata_dictionary=lambda: {
+            "description": "blah",
+            "description_content_type": "text/markdown",
+        }
     )
-    package = pretend.stub(metadata_dictionary=lambda: {
-        "description": "blah", 'description_content_type': 'text/markdown',
-    })
-    output_stream = check.StringIO()
+    output_stream = io.StringIO()
     warning_stream = ""
 
     monkeypatch.setattr(check, "_RENDERERS", {None: renderer})
-    monkeypatch.setattr(check, "_find_dists", lambda a: ["dist/dist.tar.gz"])
+    monkeypatch.setattr(commands, "_find_dists", lambda a: ["dist/dist.tar.gz"])
     monkeypatch.setattr(
-        check,
+        package_file,
         "PackageFile",
         pretend.stub(from_filename=lambda *a, **kw: package),
     )
     monkeypatch.setattr(check, "_WarningStream", lambda: warning_stream)
 
-    assert not check.check("dist/*", output_stream=output_stream)
+    assert not check.check(["dist/*"], output_stream=output_stream)
+    assert output_stream.getvalue() == "Checking dist/dist.tar.gz: PASSED\n"
+    assert renderer.render.calls == [pretend.call("blah", stream=warning_stream)]
+
+
+@pytest.mark.parametrize("content_type", ["text/plain", "text/markdown"])
+def test_check_passing_distribution_with_none_renderer(content_type, monkeypatch):
+    """Pass when rendering a content type can't fail."""
+    package = pretend.stub(
+        metadata_dictionary=lambda: {
+            "description": "blah",
+            "description_content_type": content_type,
+        }
+    )
+
+    monkeypatch.setattr(commands, "_find_dists", lambda a: ["dist/dist.tar.gz"])
+    monkeypatch.setattr(
+        package_file,
+        "PackageFile",
+        pretend.stub(from_filename=lambda *a, **kw: package),
+    )
+
+    output_stream = io.StringIO()
+    assert not check.check(["dist/*"], output_stream=output_stream)
     assert output_stream.getvalue() == "Checking dist/dist.tar.gz: PASSED\n"
-    assert renderer.render.calls == [
-        pretend.call("blah", stream=warning_stream)
-    ]
 
 
 def test_check_no_description(monkeypatch, capsys):
-    package = pretend.stub(metadata_dictionary=lambda: {
-        'description': None, 'description_content_type': None,
-    })
+    package = pretend.stub(
+        metadata_dictionary=lambda: {
+            "description": None,
+            "description_content_type": None,
+        }
+    )
 
-    monkeypatch.setattr(check, "_find_dists", lambda a: ["dist/dist.tar.gz"])
+    monkeypatch.setattr(commands, "_find_dists", lambda a: ["dist/dist.tar.gz"])
     monkeypatch.setattr(
-        check,
+        package_file,
         "PackageFile",
         pretend.stub(from_filename=lambda *a, **kw: package),
     )
 
     # used to crash with `AttributeError`
-    output_stream = check.StringIO()
-    check.check("dist/*", output_stream=output_stream)
+    output_stream = io.StringIO()
+    check.check(["dist/*"], output_stream=output_stream)
     assert output_stream.getvalue() == (
-        'Checking dist/dist.tar.gz: PASSED, with warnings\n'
-        '  warning: `long_description_content_type` missing.  '
-        'defaulting to `text/x-rst`.\n'
-        '  warning: `long_description` missing.\n'
+        "Checking dist/dist.tar.gz: PASSED, with warnings\n"
+        "  warning: `long_description_content_type` missing. "
+        "defaulting to `text/x-rst`.\n"
+        "  warning: `long_description` missing.\n"
     )
 
 
 def test_check_failing_distribution(monkeypatch):
-    renderer = pretend.stub(
-        render=pretend.call_recorder(lambda *a, **kw: None)
+    renderer = pretend.stub(render=pretend.call_recorder(lambda *a, **kw: None))
+    package = pretend.stub(
+        metadata_dictionary=lambda: {
+            "description": "blah",
+            "description_content_type": "text/markdown",
+        }
     )
-    package = pretend.stub(metadata_dictionary=lambda: {
-        "description": "blah", "description_content_type": 'text/markdown',
-    })
-    output_stream = check.StringIO()
+    output_stream = io.StringIO()
     warning_stream = "WARNING"
 
     monkeypatch.setattr(check, "_RENDERERS", {None: renderer})
-    monkeypatch.setattr(check, "_find_dists", lambda a: ["dist/dist.tar.gz"])
+    monkeypatch.setattr(commands, "_find_dists", lambda a: ["dist/dist.tar.gz"])
     monkeypatch.setattr(
-        check,
+        package_file,
         "PackageFile",
         pretend.stub(from_filename=lambda *a, **kw: package),
     )
     monkeypatch.setattr(check, "_WarningStream", lambda: warning_stream)
 
-    assert check.check("dist/*", output_stream=output_stream)
+    assert check.check(["dist/*"], output_stream=output_stream)
     assert output_stream.getvalue() == (
         "Checking dist/dist.tar.gz: FAILED\n"
         "  `long_description` has syntax errors in markup and would not be "
         "rendered on PyPI.\n"
         "    WARNING"
     )
-    assert renderer.render.calls == [
-        pretend.call("blah", stream=warning_stream)
-    ]
+    assert renderer.render.calls == [pretend.call("blah", stream=warning_stream)]
 
 
 def test_main(monkeypatch):
