@@ -12,80 +12,81 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import argparse
 import os.path
 import textwrap
-import argparse
+
+import pytest
 
 from twine import exceptions
 from twine import settings
 
-import pytest
-
 
 def test_settings_takes_no_positional_arguments():
-    """Verify that the Settings initialization is kw-only."""
+    """Raise an exception when Settings is initialized without keyword arguments."""
     with pytest.raises(TypeError):
-        settings.Settings('a', 'b', 'c')
+        settings.Settings("a", "b", "c")
 
 
-def test_settings_transforms_config(tmpdir):
-    """Verify that the settings object transforms the passed in options."""
+def test_settings_transforms_repository_config(tmpdir):
+    """Set repository config and defaults when .pypirc is provided."""
     pypirc = os.path.join(str(tmpdir), ".pypirc")
 
     with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+        fp.write(
+            textwrap.dedent(
+                """
             [pypi]
             repository: https://upload.pypi.org/legacy/
             username:username
             password:password
-        """))
+        """
+            )
+        )
     s = settings.Settings(config_file=pypirc)
-    assert (s.repository_config['repository'] ==
-            'https://upload.pypi.org/legacy/')
+    assert s.repository_config["repository"] == "https://upload.pypi.org/legacy/"
     assert s.sign is False
-    assert s.sign_with == 'gpg'
+    assert s.sign_with == "gpg"
     assert s.identity is None
-    assert s.username == 'username'
-    assert s.password == 'password'
+    assert s.username == "username"
+    assert s.password == "password"
     assert s.cacert is None
     assert s.client_cert is None
     assert s.disable_progress_bar is False
 
 
 def test_identity_requires_sign():
-    """Verify that if a user passes identity, we require sign=True."""
+    """Raise an exception when user provides identity but doesn't require sigining."""
     with pytest.raises(exceptions.InvalidSigningConfiguration):
-        settings.Settings(sign=False, identity='fakeid')
+        settings.Settings(sign=False, identity="fakeid")
 
 
 def test_password_not_required_if_client_cert(entered_password):
-    """Verify that if client_cert is provided then a password is not."""
-    test_client_cert = '/random/path'
-    settings_obj = settings.Settings(
-        username='fakeuser', client_cert=test_client_cert)
+    """Don't set password when only client_cert is provided."""
+    test_client_cert = "/random/path"
+    settings_obj = settings.Settings(username="fakeuser", client_cert=test_client_cert)
     assert not settings_obj.password
     assert settings_obj.client_cert == test_client_cert
 
 
-@pytest.mark.parametrize('client_cert', [None, ""])
+@pytest.mark.parametrize("client_cert", [None, ""])
 def test_password_is_required_if_no_client_cert(client_cert, entered_password):
-    """Verify that if client_cert is not provided then a password must be."""
-    settings_obj = settings.Settings(
-        username='fakeuser', client_cert=client_cert)
-    assert settings_obj.password == 'entered pw'
+    """Set password when client_cert is not provided."""
+    settings_obj = settings.Settings(username="fakeuser", client_cert=client_cert)
+    assert settings_obj.password == "entered pw"
 
 
 def test_client_cert_is_set_and_password_not_if_both_given(entered_password):
-    """Verify that if both password and client_cert are given they are set"""
-    client_cert = '/random/path'
+    """Set password and client_cert when both are provided."""
+    client_cert = "/random/path"
     settings_obj = settings.Settings(
-        username='fakeuser', password='anything', client_cert=client_cert)
+        username="fakeuser", password="anything", client_cert=client_cert
+    )
     assert not settings_obj.password
     assert settings_obj.client_cert == client_cert
 
 
 class TestArgumentParsing:
-
     @staticmethod
     def parse_args(args):
         parser = argparse.ArgumentParser()
@@ -93,7 +94,7 @@ class TestArgumentParsing:
         return parser.parse_args(args)
 
     def test_non_interactive_flag(self):
-        args = self.parse_args(['--non-interactive'])
+        args = self.parse_args(["--non-interactive"])
         assert args.non_interactive
 
     def test_non_interactive_environment(self, monkeypatch):
