@@ -15,26 +15,32 @@
 import os.path
 import textwrap
 
-import pytest
 import pretend
+import pytest
+import requests
 
-from twine import exceptions, utils
+from twine import exceptions
+from twine import utils
 
-import helpers
+from . import helpers
 
 
 def test_get_config(tmpdir):
     pypirc = os.path.join(str(tmpdir), ".pypirc")
 
     with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+        fp.write(
+            textwrap.dedent(
+                """
             [distutils]
             index-servers = pypi
 
             [pypi]
             username = testuser
             password = testpassword
-        """))
+        """
+            )
+        )
 
     assert utils.get_config(pypirc) == {
         "pypi": {
@@ -46,18 +52,19 @@ def test_get_config(tmpdir):
 
 
 def test_get_config_no_distutils(tmpdir):
-    """
-    Even if the user hasn't set PyPI has an index server
-    in 'index-servers', default to uploading to PyPI.
-    """
+    """Upload by default to PyPI if an index server is not set in .pypirc."""
     pypirc = os.path.join(str(tmpdir), ".pypirc")
 
     with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+        fp.write(
+            textwrap.dedent(
+                """
             [pypi]
             username = testuser
             password = testpassword
-        """))
+        """
+            )
+        )
 
     assert utils.get_config(pypirc) == {
         "pypi": {
@@ -77,14 +84,18 @@ def test_get_config_no_section(tmpdir):
     pypirc = os.path.join(str(tmpdir), ".pypirc")
 
     with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+        fp.write(
+            textwrap.dedent(
+                """
             [distutils]
             index-servers = pypi foo
 
             [pypi]
             username = testuser
             password = testpassword
-        """))
+        """
+            )
+        )
 
     assert utils.get_config(pypirc) == {
         "pypi": {
@@ -99,12 +110,16 @@ def test_get_config_override_pypi_url(tmpdir):
     pypirc = os.path.join(str(tmpdir), ".pypirc")
 
     with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+        fp.write(
+            textwrap.dedent(
+                """
             [pypi]
             repository = http://pypiproxy
-        """))
+        """
+            )
+        )
 
-    assert utils.get_config(pypirc)['pypi']['repository'] == 'http://pypiproxy'
+    assert utils.get_config(pypirc)["pypi"]["repository"] == "http://pypiproxy"
 
 
 def test_get_config_missing(tmpdir):
@@ -119,29 +134,30 @@ def test_get_config_missing(tmpdir):
         "testpypi": {
             "repository": utils.TEST_REPOSITORY,
             "username": None,
-            "password": None
+            "password": None,
         },
     }
 
 
 def test_empty_userpass(tmpdir):
-    """
-    Empty username and password may be supplied to suppress
-    prompts. See #426.
-    """
+    """Suppress prompts if empty username and password are provided in .pypirc."""
     pypirc = os.path.join(str(tmpdir), ".pypirc")
 
     with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+        fp.write(
+            textwrap.dedent(
+                """
             [pypi]
             username=
             password=
-        """))
+        """
+            )
+        )
 
     config = utils.get_config(pypirc)
-    pypi = config['pypi']
+    pypi = config["pypi"]
 
-    assert pypi['username'] == pypi['password'] == ''
+    assert pypi["username"] == pypi["password"] == ""
 
 
 def test_get_repository_config_missing(tmpdir):
@@ -153,22 +169,48 @@ def test_get_repository_config_missing(tmpdir):
         "username": None,
         "password": None,
     }
-    assert (utils.get_repository_from_config(pypirc, 'foo', repository_url) ==
-            exp)
-    assert (utils.get_repository_from_config(pypirc, 'pypi', repository_url) ==
-            exp)
+    assert utils.get_repository_from_config(pypirc, "foo", repository_url) == exp
+    assert utils.get_repository_from_config(pypirc, "pypi", repository_url) == exp
     exp = {
-            "repository": utils.DEFAULT_REPOSITORY,
-            "username": None,
-            "password": None,
-        }
+        "repository": utils.DEFAULT_REPOSITORY,
+        "username": None,
+        "password": None,
+    }
     assert utils.get_repository_from_config(pypirc, "pypi") == exp
 
 
+@pytest.mark.parametrize(
+    "repo_url, message",
+    [
+        (
+            "ftp://test.pypi.org",
+            r"scheme was required to be one of \['http', 'https'\]",
+        ),
+        ("https:/", "host was required but missing."),
+        ("//test.pypi.org", "scheme was required but missing."),
+        ("foo.bar", "host, scheme were required but missing."),
+    ],
+)
+def test_get_repository_config_with_invalid_url(tmpdir, repo_url, message):
+    """Raise an exception for a URL with an invalid/missing scheme and/or host."""
+    pypirc = os.path.join(str(tmpdir), ".pypirc")
+
+    with pytest.raises(
+        exceptions.UnreachableRepositoryURLDetected, match=message,
+    ):
+        utils.get_repository_from_config(pypirc, "pypi", repo_url)
+
+
+def test_get_repository_config_missing_config(tmpdir):
+    """Raise an exception when a repository isn't defined in .pypirc."""
+    pypirc = os.path.join(str(tmpdir), ".pypirc")
+    with pytest.raises(exceptions.InvalidConfiguration):
+        utils.get_repository_from_config(pypirc, "foobar")
+
+
 def test_get_config_deprecated_pypirc():
     tests_dir = os.path.dirname(os.path.abspath(__file__))
-    deprecated_pypirc_path = os.path.join(tests_dir, 'fixtures',
-                                          'deprecated-pypirc')
+    deprecated_pypirc_path = os.path.join(tests_dir, "fixtures", "deprecated-pypirc")
 
     assert utils.get_config(deprecated_pypirc_path) == {
         "pypi": {
@@ -185,11 +227,11 @@ def test_get_config_deprecated_pypirc():
 
 
 @pytest.mark.parametrize(
-    ('cli_value', 'config', 'key', 'strategy', 'expected'),
+    ("cli_value", "config", "key", "strategy", "expected"),
     (
-        ('cli', {}, 'key', lambda: 'fallback', 'cli'),
-        (None, {'key': 'value'}, 'key', lambda: 'fallback', 'value'),
-        (None, {}, 'key', lambda: 'fallback', 'fallback'),
+        ("cli", {}, "key", lambda: "fallback", "cli"),
+        (None, {"key": "value"}, "key", lambda: "fallback", "value"),
+        (None, {}, "key", lambda: "fallback", "fallback"),
     ),
 )
 def test_get_userpass_value(cli_value, config, key, strategy, expected):
@@ -198,39 +240,70 @@ def test_get_userpass_value(cli_value, config, key, strategy, expected):
 
 
 @pytest.mark.parametrize(
-    ('env_name', 'default', 'environ', 'expected'),
+    ("env_name", "default", "environ", "expected"),
     [
-        ('MY_PASSWORD', None, {}, None),
-        ('MY_PASSWORD', None, {'MY_PASSWORD': 'foo'}, 'foo'),
-        ('URL', 'https://example.org', {}, 'https://example.org'),
-        ('URL', 'https://example.org', {'URL': 'https://pypi.org'},
-            'https://pypi.org'),
+        ("MY_PASSWORD", None, {}, None),
+        ("MY_PASSWORD", None, {"MY_PASSWORD": "foo"}, "foo"),
+        ("URL", "https://example.org", {}, "https://example.org"),
+        ("URL", "https://example.org", {"URL": "https://pypi.org"}, "https://pypi.org"),
     ],
 )
 def test_default_to_environment_action(env_name, default, environ, expected):
-    option_strings = ('-x', '--example')
-    dest = 'example'
+    option_strings = ("-x", "--example")
+    dest = "example"
     with helpers.set_env(**environ):
         action = utils.EnvironmentDefault(
-            env=env_name,
-            default=default,
-            option_strings=option_strings,
-            dest=dest,
+            env=env_name, default=default, option_strings=option_strings, dest=dest,
         )
     assert action.env == env_name
     assert action.default == expected
 
 
-@pytest.mark.parametrize('repo_url', [
-    "https://pypi.python.org",
-    "https://testpypi.python.org"
-])
+@pytest.mark.parametrize(
+    "repo_url", ["https://pypi.python.org", "https://testpypi.python.org"]
+)
 def test_check_status_code_for_deprecated_pypi_url(repo_url):
-    response = pretend.stub(
-        status_code=410,
-        url=repo_url
-    )
+    response = pretend.stub(status_code=410, url=repo_url)
 
     # value of Verbose doesn't matter for this check
     with pytest.raises(exceptions.UploadToDeprecatedPyPIDetected):
         utils.check_status_code(response, False)
+
+
+@pytest.mark.parametrize(
+    "repo_url", ["https://pypi.python.org", "https://testpypi.python.org"],
+)
+def test_check_status_code_for_missing_status_code(capsys, repo_url):
+    """Print HTTP errors based on verbosity level."""
+    response = pretend.stub(
+        status_code=403,
+        url=repo_url,
+        raise_for_status=pretend.raiser(requests.HTTPError),
+        text="Forbidden",
+    )
+
+    with pytest.raises(requests.HTTPError):
+        utils.check_status_code(response, True)
+
+    # Different messages are printed based on the verbose level
+    captured = capsys.readouterr()
+    assert captured.out == "Content received from server:\nForbidden\n"
+
+    with pytest.raises(requests.HTTPError):
+        utils.check_status_code(response, False)
+
+    captured = capsys.readouterr()
+    assert captured.out == "NOTE: Try --verbose to see response content.\n"
+
+
+@pytest.mark.parametrize(
+    ("size_in_bytes, formatted_size"),
+    [(3704, "3.6 KB"), (1153433, "1.1 MB"), (21412841, "20.4 MB")],
+)
+def test_get_file_size(size_in_bytes, formatted_size, monkeypatch):
+    """Get the size of file as a string with units."""
+    monkeypatch.setattr(os.path, "getsize", lambda _: size_in_bytes)
+
+    file_size = utils.get_file_size(size_in_bytes)
+
+    assert file_size == formatted_size
