@@ -11,17 +11,24 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from typing import Callable, DefaultDict, Dict, Optional
-
-import os
-import os.path
-import functools
 import argparse
 import collections
 import configparser
-from urllib.parse import urlparse, urlunparse
+import functools
+import os
+import os.path
+from typing import Any
+from typing import Callable
+from typing import DefaultDict
+from typing import Dict
+from typing import Optional
+from typing import Sequence
+from typing import Union
+from urllib.parse import urlparse
+from urllib.parse import urlunparse
 
 import requests
+import rfc3986
 
 from twine import exceptions
 
@@ -59,7 +66,7 @@ def get_config(path: str = "~/.pypirc") -> Dict[str, RepositoryConfig]:
         parser.read(path)
 
         # Get a list of index_servers from the config file
-        # format: https://docs.python.org/3/distutils/packageindex.html#pypirc
+        # format: https://packaging.python.org/specifications/pypirc/
         if parser.has_option("distutils", "index-servers"):
             index_servers = parser.get("distutils", "index-servers").split()
 
@@ -67,8 +74,9 @@ def get_config(path: str = "~/.pypirc") -> Dict[str, RepositoryConfig]:
             if parser.has_option("server-login", key):
                 defaults[key] = parser.get("server-login", key)
 
-    config: DefaultDict[str, RepositoryConfig] = \
-        collections.defaultdict(lambda: defaults.copy())
+    config: DefaultDict[str, RepositoryConfig] = collections.defaultdict(
+        lambda: defaults.copy()
+    )
 
     # don't require users to manually configure URLs for these repositories
     config["pypi"]["repository"] = DEFAULT_REPOSITORY
@@ -78,8 +86,11 @@ def get_config(path: str = "~/.pypirc") -> Dict[str, RepositoryConfig]:
     # optional configuration values for individual repositories
     for repository in index_servers:
         for key in [
-            "username", "repository", "password",
-            "ca_cert", "client_cert",
+            "username",
+            "repository",
+            "password",
+            "ca_cert",
+            "client_cert",
         ]:
             if parser.has_option(repository, key):
                 config[repository][key] = parser.get(repository, key)
@@ -89,42 +100,56 @@ def get_config(path: str = "~/.pypirc") -> Dict[str, RepositoryConfig]:
     return dict(config)
 
 
+def _validate_repository_url(repository_url: str) -> None:
+    """Validate the given url for allowed schemes and components."""
+    # Allowed schemes are http and https, based on whether the repository
+    # supports TLS or not, and scheme and host must be present in the URL
+    validator = (
+        rfc3986.validators.Validator()
+        .allow_schemes("http", "https")
+        .require_presence_of("scheme", "host")
+    )
+    try:
+        validator.validate(rfc3986.uri_reference(repository_url))
+    except rfc3986.exceptions.RFC3986Exception as exc:
+        raise exceptions.UnreachableRepositoryURLDetected(
+            f"Invalid repository URL: {exc.args[0]}."
+        )
+
+
 def get_repository_from_config(
-    config_file: str,
-    repository: str,
-    repository_url: Optional[str] = None
+    config_file: str, repository: str, repository_url: Optional[str] = None
 ) -> RepositoryConfig:
     # Get our config from, if provided, command-line values for the
     # repository name and URL, or the .pypirc file
-    if repository_url and "://" in repository_url:
+
+    if repository_url:
+        _validate_repository_url(repository_url)
         # prefer CLI `repository_url` over `repository` or .pypirc
         return {
             "repository": repository_url,
             "username": None,
             "password": None,
         }
-    if repository_url and "://" not in repository_url:
-        raise exceptions.UnreachableRepositoryURLDetected(
-            "Repository URL {} has no protocol. Please add "
-            "'https://'. \n".format(repository_url))
     try:
         return get_config(config_file)[repository]
     except KeyError:
         msg = (
             "Missing '{repo}' section from the configuration file\n"
             "or not a complete URL in --repository-url.\n"
-            "Maybe you have a out-dated '{cfg}' format?\n"
+            "Maybe you have an out-dated '{cfg}' format?\n"
             "more info: "
-            "https://docs.python.org/distutils/packageindex.html#pypirc\n"
-        ).format(
-            repo=repository,
-            cfg=config_file
-        )
+            "https://packaging.python.org/specifications/pypirc/\n"
+        ).format(repo=repository, cfg=config_file)
         raise exceptions.InvalidConfiguration(msg)
 
 
-_HOSTNAMES = {"pypi.python.org", "testpypi.python.org", "upload.pypi.org",
-              "test.pypi.org"}
+_HOSTNAMES = {
+    "pypi.python.org",
+    "testpypi.python.org",
+    "upload.pypi.org",
+    "test.pypi.org",
+}
 
 
 def normalize_repository_url(url: str) -> str:
@@ -134,6 +159,18 @@ def normalize_repository_url(url: str) -> str:
     return urlunparse(parsed)
 
 
+def get_file_size(filename: str) -> str:
+    """Return the size of a file in KB, or MB if >= 1024 KB."""
+    file_size = os.path.getsize(filename) / 1024
+    size_unit = "KB"
+
+    if file_size > 1024:
+        file_size = file_size / 1024
+        size_unit = "MB"
+
+    return f"{file_size:.1f} {size_unit}"
+
+
 def check_status_code(response: requests.Response, verbose: bool) -> None:
     """Generate a helpful message based on the response from the repository.
 
@@ -149,23 +186,24 @@ def check_status_code(response: requests.Response, verbose: bool) -> None:
             f"pypi.org and test.pypi.org. Try using {DEFAULT_REPOSITORY} (or "
             f"{TEST_REPOSITORY}) to upload your packages instead. These are "
             f"the default URLs for Twine now. More at "
-            f"https://packaging.python.org/guides/migrating-to-pypi-org/.")
+            f"https://packaging.python.org/guides/migrating-to-pypi-org/."
+        )
     elif response.status_code == 405 and "pypi.org" in response.url:
         raise exceptions.InvalidPyPIUploadURL(
             f"It appears you're trying to upload to pypi.org but have an "
             f"invalid URL. You probably want one of these two URLs: "
             f"{DEFAULT_REPOSITORY} or {TEST_REPOSITORY}. Check your "
-            f"--repository-url value.")
+            f"--repository-url value."
+        )
 
     try:
         response.raise_for_status()
     except requests.HTTPError as err:
         if response.text:
             if verbose:
-                print('Content received from server:\n{}'.format(
-                    response.text))
+                print("Content received from server:\n{}".format(response.text))
             else:
-                print('NOTE: Try --verbose to see response content.')
+                print("NOTE: Try --verbose to see response content.")
         raise err
 
 
@@ -173,9 +211,9 @@ def get_userpass_value(
     cli_value: Optional[str],
     config: RepositoryConfig,
     key: str,
-    prompt_strategy: Optional[Callable] = None
+    prompt_strategy: Optional[Callable[[], str]] = None,
 ) -> Optional[str]:
-    """Gets the username / password from config.
+    """Get the username / password from config.
 
     Uses the following rules:
 
@@ -205,14 +243,8 @@ def get_userpass_value(
         return None
 
 
-get_cacert = functools.partial(
-    get_userpass_value,
-    key='ca_cert',
-)
-get_clientcert = functools.partial(
-    get_userpass_value,
-    key='client_cert',
-)
+get_cacert = functools.partial(get_userpass_value, key="ca_cert")
+get_clientcert = functools.partial(get_userpass_value, key="client_cert")
 
 
 class EnvironmentDefault(argparse.Action):
@@ -223,7 +255,7 @@ class EnvironmentDefault(argparse.Action):
         env: str,
         required: bool = True,
         default: Optional[str] = None,
-        **kwargs
+        **kwargs: Any,
     ) -> None:
         default = os.environ.get(env, default)
         self.env = env
@@ -231,29 +263,35 @@ class EnvironmentDefault(argparse.Action):
             required = False
         super().__init__(default=default, required=required, **kwargs)
 
-    def __call__(self, parser, namespace, values, option_string=None):
+    def __call__(
+        self,
+        parser: argparse.ArgumentParser,
+        namespace: argparse.Namespace,
+        values: Union[str, Sequence[Any], None],
+        option_string: Optional[str] = None,
+    ) -> None:
         setattr(namespace, self.dest, values)
 
 
 class EnvironmentFlag(argparse.Action):
     """Set boolean flag from environment variable."""
 
-    def __init__(
-        self,
-        env: str,
-        **kwargs
-    ) -> None:
+    def __init__(self, env: str, **kwargs: Any) -> None:
         default = self.bool_from_env(os.environ.get(env))
         self.env = env
         super().__init__(default=default, nargs=0, **kwargs)
 
-    def __call__(self, parser, namespace, values, option_string=None):
+    def __call__(
+        self,
+        parser: argparse.ArgumentParser,
+        namespace: argparse.Namespace,
+        values: Union[str, Sequence[Any], None],
+        option_string: Optional[str] = None,
+    ) -> None:
         setattr(namespace, self.dest, True)
 
     @staticmethod
-    def bool_from_env(val):
-        """
-        Allow '0' and 'false' and 'no' to be False
-        """
-        falsey = {'0', 'false', 'no'}
-        return val and val.lower() not in falsey
+    def bool_from_env(val: Optional[str]) -> bool:
+        """Allow '0' and 'false' and 'no' to be False."""
+        falsey = {"0", "false", "no"}
+        return bool(val and val.lower() not in falsey)
